import csv
import re
# latitude and longitude values are from https://www.geodatos.net/en

# https://www.geeksforgeeks.org/pandas/reading-csv-files-in-python/
# https://docs.python.org/3/library/csv.html


def input_validator(row):

    try:
        temp_customer = row["customer"]
        if len(temp_customer) == 0 or temp_customer.isprintable() == False:
            print("customer name not valid")
            return False

        try:
            temp_latitude = float(row["latitude"])
        except Exception as e:
            print("Latitude must be a number")
            print(e)
            return False

        if  temp_latitude < -90 or temp_latitude > 90:
            print("Latitude out of range, it must be between -90 and 90")
            return False

        try:
            temp_longitude = float(row["longitude"])
        except Exception as e:
            print("Longitude must be a number")
            print(e)
            return False
        if temp_longitude <-180 or temp_longitude > 180:
            print("Longitude out of range, it must be between -180 and 180")
            return False

        temp_priority = row["priority"].lower()
        if not re.match(r'^(low|medium|high)$', temp_priority):
            print("Priority must be low or medium or high")
            return False

        try:
            temp_weight = float(row["weight_kg"])
        except Exception as e:
            print("Weight must be a number")
            print(e)
            return False
        if temp_weight < 0:
            print("Weight must be positive")
            return False

    except Exception as e:
        print("Something is unexpectedly wrong, see following error")
        print(e)
        print()
        return False

    else:
        return True

def rejector(rejected_row):

    duplicate_checker = False
    try:
        with open('rejected.csv', mode='r') as file:
            reader = csv.DictReader(file)
            for row in reader:
                if rejected_row == row:
                    print("Rejected row already in rejected list")
                    print()
                    duplicate_checker = True
                    break

        if not duplicate_checker:
            fieldnames = ['customer', 'latitude', 'longitude', 'priority', 'weight_kg']
            with open('rejected.csv', 'a', newline='') as csvfile:
                writer = csv.DictWriter(csvfile, fieldnames=fieldnames)

                writer.writerow(rejected_row)
                print("Rejected row added in rejected list")
                print()

    except FileNotFoundError:
        open('rejected.csv', 'w')
        rejector(rejected_row)


def input_processor():
    customer_dictionary = {}
    try:
        with open('input.csv', mode='r') as file:
            reader = csv.DictReader(file)
            row_counter = 0

            for row in reader:
                row_counter += 1
                if input_validator(row):
                    customer_dictionary["stop " + str(row_counter)] = row
                else:
                    print("Row " + str(row_counter) + " is Invalid, adding row to rejected.csv")
                    rejector(row)
    except FileNotFoundError:
        open('input.csv', 'w')
        input_processor()

    return customer_dictionary


from . import optimizer

print("\nCourier Optimizer module begun")
print("Input a depot, then create the route based on chosen mode and criterion")
print("Customer information is stored within input.csv")
while True:
    print("\nEnter the number of the option to continue")
    print("1. Input Depot")
    print("2. Start Route")
    print("3. Quit")
    answer = input()

    # default values in case user goes straight to start route
    inputted_depot = {'customer': 'near oslo', 'latitude': '60',
                             'longitude': '11', 'priority': 'medium', 'weight_kg': '0'}

    if answer == "1":
        print("Provide name of location of the Depot")
        print("Default name is: near oslo")
        inputted_depot['customer'] = input()

        print("Provide latitude of the Depot")
        print("Default latitude is: 60")
        inputted_depot['latitude'] = input()

        print("Provide longitude of the Depot")
        print("Default longitude is: 11")
        inputted_depot['longitude'] = input()

    elif answer == "2":
        modeValidity = False
        while not modeValidity:
            print("Provide chosen mode")
            print("options are: car, bicycle, walking")
            mode = input()
            if not re.match(r'^(car|bicycle|walking)$', mode):
                print("Must input a valid mode")
            else:
                modeValidity = True

        criterionValidity = False
        while not criterionValidity:
            print("Provide chosen criterion")
            print("options are:")
            print("1 for fastest total time")
            print("2 lowest total cost")
            print("3 for lowest total CO2 emission")
            criterion = input()
            if not re.match(r'^([123])$', criterion):
                print("Must input a valid criterion")
            else:
                criterionValidity = True

        optimizer.route_creator(mode, criterion, inputted_depot)
    elif answer == "3":
        exit(0)
    else:
        print("Invalid input, try again")


import csv

def outputter(mode, criterion, tsp_route, graph, speed, cost_per_km, co2_per_km):

    if mode == "car":
        print("Totals for route with car:")
    elif mode == "bicycle":
        print("Totals for route with bicycle:")
    elif mode == "walking":
        print("Totals for route with walking:")


    print("Delivery Route:", " -> ".join(tsp_route))

    total_distance = 0


    fieldnames = ['Stop', 'Distance from last stop', 'Total distance', 'ETA', 'cost','CO2']
    with open('route.csv', 'w', newline='') as csvfile:
        csvfile.truncate(0)
        writer = csv.DictWriter(csvfile, fieldnames=fieldnames)
        writer.writeheader()
        output_row = {'Stop': '', 'Distance from last stop':'','Total distance':'', 'ETA': '', 'cost': '', 'CO2':''}

        total_distance = 0
        total_time = 0
        total_cost = 0
        total_co2 = 0

        for i in range((len(tsp_route))):
            if i == 0:
                output_row['Stop'] = tsp_route[0]
                output_row['Distance from last stop'] = "0"
                output_row['Total distance'] = "0"
                output_row['ETA'] = "0"
                output_row['cost'] = "0"
                output_row['CO2'] = "0"
                writer.writerow(output_row)
                continue
            output_row['Stop'] = tsp_route[i]
            distance_from_last_stop = graph[tsp_route[i-1]][tsp_route[i]]["distance"]
            output_row['Distance from last stop'] = str(distance_from_last_stop)
            total_distance += distance_from_last_stop
            output_row['Total distance'] = str(total_distance)
            total_time = total_distance / speed
            output_row['ETA'] = str(total_time)
            total_cost = total_distance * cost_per_km
            output_row['cost'] = str(total_cost)
            total_co2 = total_distance * co2_per_km
            output_row['CO2'] = str(total_co2)

            writer.writerow(output_row)

    #for i in range(len(tsp_route) - 1):
    #    total_distance += graph[tsp_route[i]][tsp_route[i + 1]]["distance"]

    print(f"Total Distance: {total_distance:.2f} km")


    if criterion == "1":
        print(f"Total Time: {total_time:.2f} hours")
    elif criterion == "2":
        print(f"Total cost: {total_cost:.2f} NOK")
    elif criterion == "3":
        print(f"Total CO2: {total_co2:.2f} grams of CO2")
    print("Check route.csv within the output folder for more details")

